/* Includes *****************************************************************/

#include "strobelight.h"
#include "api_amd.h"
#include "api_nvidia.h"
#include "debug.h"
#include "desktop.h"
#include "displays.h"
#include "hotkeys.h"
#include "icon.h"
#include "init_amd.h"
#include "init_nvidia.h"
#include "menu.h"
#include "options.h"
#include "strobing.h"
#include "vcp.h"

/* Functions ****************************************************************/

BOOL open_link(HWND window, LPCTSTR link)
{
	ShellExecute(window, _T("open"), link, NULL, NULL, SW_SHOW);
	return TRUE;
}

/****************************************************************************/

BOOL CALLBACK main_window(HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
	static BOOL amd;
	static BOOL nvidia;
	static struct display_list *display_list;
	static BOOL strobed;

	switch (message)
	{
		case WM_INITDIALOG:
			amd = load_amd();
			nvidia = load_nvidia();

			if (!amd && !nvidia)
			{
				debug_printf("INIT: No compatible video drivers found.");
				MessageBox(NULL, _T("No compatible video drivers found."), STROBELIGHT_VERSION, MB_ICONERROR);
				PostQuitMessage(1);
				return FALSE;
			}

			display_list = new_display_list();

			if (!skip_init)
			{
				while (display_list->count == 0)
				{
					if (amd)
						init_amd(display_list, window);

					if (nvidia)
						init_nvidia(display_list, window);

					ShowWindow(window, SW_HIDE);

					if (display_list->count == 0)
					{
						debug_printf("INIT: No compatible displays found.");

						if (MessageBox(NULL, _T("No compatible displays found."), STROBELIGHT_VERSION, MB_ICONERROR | MB_RETRYCANCEL) != IDRETRY)
						{
							PostQuitMessage(1);
							return FALSE;
						}
					}
				}
			}

			while (!wait_for_desktop())
			{
				debug_printf("INIT: Failed to open desktop.");

				if (MessageBox(NULL, _T("Failed to open desktop."), STROBELIGHT_VERSION, MB_ICONERROR | MB_RETRYCANCEL) != IDRETRY)
				{
					PostQuitMessage(1);
					return FALSE;
				}
			}

			if (!load_strobing_displays())
			{
				debug_printf("INIT: No strobed refresh rates found.");
				MessageBox(NULL, _T("No strobed refresh rates found."), STROBELIGHT_VERSION, MB_ICONERROR);
				PostQuitMessage(1);
				return FALSE;
			}

			if (display_list->restart || start_strobed && is_any_strobing_display_not_strobed() || !start_strobed && is_any_switchable_display_strobed())
			{
				SendMessage(window, WM_SYSCOMMAND, SC_MONITORPOWER, 2);
				Sleep(1000);
				keybd_event(0, 0, 0, 0);
				mouse_event(0, 0, 0, 0, 0);
				set_strobed(start_strobed);
			}

			if (init_only)
			{
				PostQuitMessage(0);
				return FALSE;
			}

			load_notify_menu();
			register_hotkeys(window);
			strobed = get_strobed();
			add_notify_icon(window, strobed);
			return FALSE;		// Return FALSE to avoid activating the window so hotkeys work properly

		case WM_DISPLAYCHANGE:
			// Do not process resolution changes during initialization
			if (IsWindowVisible(window))
				return FALSE;

			strobed = get_strobed();
			update_notify_icon(strobed);
			return TRUE;

		case WM_HOTKEY:
			switch (wParam)
			{
				case IDM_STROBING_ON:
					if (is_any_switchable_display_strobed())
						set_strobed(FALSE);

					set_strobed(TRUE);
					break;

				case IDM_STROBING_OFF:
					if (is_any_switchable_display_not_strobed())
						set_strobed(TRUE);

					set_strobed(FALSE);
					break;

				case IDM_BRIGHTNESS_10:
				case IDM_BRIGHTNESS_20:
				case IDM_BRIGHTNESS_30:
				case IDM_BRIGHTNESS_40:
				case IDM_BRIGHTNESS_50:
				case IDM_BRIGHTNESS_60:
				case IDM_BRIGHTNESS_70:
				case IDM_BRIGHTNESS_80:
				case IDM_BRIGHTNESS_90:
				case IDM_BRIGHTNESS_100:
					set_brightness(display_list, wParam % 100, strobed);
					break;

				default:
					if (wParam > IDM_FREQUENCY)
					{
						set_frequency(wParam % 1000);
						break;
					}

					return FALSE;
			}

			flush_hotkeys();
			return TRUE;

		case WM_APP:
			switch (lParam)
			{
				case WM_RBUTTONUP:
					show_notify_menu(window, strobed);
					return TRUE;

				default:
					return FALSE;
			}

		case WM_COMMAND:
			switch (wParam)
			{
				case IDCANCEL:
					// Do not process the ESC key after initialization
					if (!IsWindowVisible(window))
						return FALSE;

					debug_printf("INIT: CANCEL");
					ExitProcess(0);

				case IDM_REINITIALIZE:
					delete_notify_icon();
					unregister_hotkeys();
					flush_hotkeys();
					unload_notify_menu();
					unload_strobing_displays();
					delete_display_list(display_list);
					display_list = new_display_list();

					while (display_list->count == 0)
					{
						if (amd)
						{
							refresh_amd();
							init_amd(display_list, window);
						}

						if (nvidia)
							init_nvidia(display_list, window);

						ShowWindow(window, SW_HIDE);

						if (display_list->count == 0)
						{
							debug_printf("INIT: No compatible displays found.");

							if (MessageBox(NULL, _T("No compatible displays found."), STROBELIGHT_VERSION, MB_ICONERROR | MB_RETRYCANCEL) != IDRETRY)
							{
								PostQuitMessage(1);
								return FALSE;
							}
						}
					}

					if (!load_strobing_displays())
					{
						debug_printf("INIT: No strobed refresh rates found.");
						MessageBox(NULL, _T("No strobed refresh rates found."), STROBELIGHT_VERSION, MB_ICONERROR);
						PostQuitMessage(1);
						return FALSE;
					}

					SendMessage(window, WM_SYSCOMMAND, SC_MONITORPOWER, 2);
					Sleep(1000);
					keybd_event(0, 0, 0, 0);
					mouse_event(0, 0, 0, 0, 0);
					set_strobed(start_strobed);
					load_notify_menu();
					register_hotkeys(window);
					strobed = get_strobed();
					add_notify_icon(window, strobed);
					return TRUE;

				case IDM_STROBING_ON:
					if (is_any_switchable_display_strobed())
						set_strobed(FALSE);

					set_strobed(TRUE);
					return TRUE;

				case IDM_STROBING_OFF:
					if (is_any_switchable_display_not_strobed())
						set_strobed(TRUE);

					set_strobed(FALSE);
					return TRUE;

				case IDM_BRIGHTNESS_10:
				case IDM_BRIGHTNESS_20:
				case IDM_BRIGHTNESS_30:
				case IDM_BRIGHTNESS_40:
				case IDM_BRIGHTNESS_50:
				case IDM_BRIGHTNESS_60:
				case IDM_BRIGHTNESS_70:
				case IDM_BRIGHTNESS_80:
				case IDM_BRIGHTNESS_90:
				case IDM_BRIGHTNESS_100:
					set_brightness(display_list, wParam % 100, strobed);
					return TRUE;

				case IDM_STROBELIGHT_LINK:
					open_link(window, _T("http://www.monitortests.com/strobelight"));
					return TRUE;

				case IDM_BLURBUSTERS_LINK:
					open_link(window, _T("http://www.blurbusters.com/"));
					return TRUE;

				case IDM_TESTUFO_LINK:
					open_link(window, _T("http://www.testufo.com/strobelight"));
					return TRUE;

				case IDM_EXIT:
					delete_notify_icon();
					unregister_hotkeys();
					flush_hotkeys();
					unload_notify_menu();
					unload_strobing_displays();
					delete_display_list(display_list);
					PostQuitMessage(0);
					return TRUE;

				default:
					if (wParam > IDM_FREQUENCY)
					{
						set_frequency(wParam % 1000);
						return TRUE;
					}

					return FALSE;
			}
	}

	return FALSE;
}

/* Main *********************************************************************/

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	HWND window;
	MSG message;

	CreateMutex(NULL, TRUE, _T("4826d5799955b86018ace7f5cf6df1cc"));

	if (GetLastError() == ERROR_ALREADY_EXISTS)
		return 1;

	parse_options();
	window = CreateDialog(hInstance, _T("IDD_MAIN"), NULL, main_window);

	if (!window)
		return 1;

	while (GetMessage(&message, NULL, 0, 0) > 0)
	{
		if (!IsDialogMessage(window, &message))
		{
			TranslateMessage(&message);
			DispatchMessage(&message);
		}
	}

	DestroyWindow(window);
	return message.wParam;
}
